-- Rationalism_Opener.lua
-- Author: Machiavelli
-- DateCreated: 6/2/2012 7:42:07 PM
--------------------------------------------------------------
function ScienceAndCultureOnPopulationGrowth(hexX, hexY, population, citySize)
	local plot = Map.GetPlot(ToGridFromHex(hexX, hexY));
	local city = plot:GetPlotCity();
	local player = Players[city:GetOwner()];
	local policyID = GameInfoTypes["POLICY_RATIONALISM"];

	-- Only run this if the player has the Rationalism opener policy and the population is larger than the city has been and the population is greater than 10
	if(player:HasPolicy(policyID) and not player:IsPolicyBlocked(policyID) and city:GetHighestPopulation() <= population) then
		local bonus = 0;
		local universityType = GetBuildingTypeFromClass("BUILDINGCLASS_UNIVERSITY", player:GetCivilizationType());
		local publicSchoolType = GetBuildingTypeFromClass("BUILDINGCLASS_PUBLIC_SCHOOL", player:GetCivilizationType());
		
		if(universityType ~= nil and city:IsHasBuilding(GameInfoTypes[universityType])) then
			bonus = 10;
			if(publicSchoolType ~= nil and city:IsHasBuilding(GameInfoTypes[publicSchoolType])) then
				bonus = 25;
			end
		end

		if(bonus > 0) then
			-- Handle the Science --------------
			local team = Teams[player:GetTeam()];
			local teamTechs = team:GetTeamTechs();
			local currentTech = GetCurrentOrLegalTech(city:GetOwner());

			local adjustedBonusScience = 0;

			-- Only provide Science if a valid tech was found
			if (currentTech >= 0) then
				adjustedBonusScience = (bonus * GameInfo.GameSpeeds[Game:GetGameSpeedType()].ResearchPercent) / 100;
				-- Give the bonus science
				teamTechs:ChangeResearchProgress(currentTech, adjustedBonus, city:GetOwner());
			end

			-- Handle the Culture --------------
			local adjustedBonusCulture = (bonus * GameInfo.GameSpeeds[Game:GetGameSpeedType()].CulturePercent) / 100;
			player:ChangeJONSCulture(adjustedBonusCulture);

			-- Send a notification to the player --------------
			local text = Locale.ConvertTextKey("TXT_KEY_SCIENCE_AND_CULTURE_FROM_CITY_GROWTH", tostring(adjustedBonusScience), tostring(adjustedBonusCulture), city:GetName());
			player:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, text, text);
		end
	end
end
Events.SerialEventCityPopulationChanged.Add(ScienceAndCultureOnPopulationGrowth);

--------------------
-- Private helper functions
--------------------
function GetBuildingTypeFromClass(buildingClass, civilizationTypeID)
	-- Assume it is the default building
	local buildingType = GameInfo.BuildingClasses[buildingClass].DefaultBuilding;

	-- See if this civilization has a unique building for this building class
	local condition = "CivilizationType = '" .. GameInfo.Civilizations[civilizationTypeID].Type .. "' AND BuildingClassType = '" .. buildingClass .. "'";
	for row in GameInfo.Civilization_BuildingClassOverrides(condition) do
		buildingType = row.BuildingType;
		return buildingType;
	end

	return buildingType;
end

function GetCurrentOrLegalTech(playerID)
	local player = Players[playerID];
	
	if(player:GetCurrentResearch() ~= nil and player:GetCurrentResearch() >= 0) then
		return player:GetCurrentResearch();
	end
	-- Find a valid tech and return that
	for row in GameInfo.Technologies() do
		if(player:CanResearch(row.ID)) then
			return row.ID;
		end
	end
	-- Could not find a valid tech
	return -1;
end